VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Piece"
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Private m_typePiece As Byte
Private m_isWhite As Boolean
Private m_square As Square
Private m_squareInitial As Square
Private m_player As Player
Private m_opponent As Player
Private m_board As Board

Private m_LastMoveTurnNo As Integer
Private m_TurnNoPromotion As Integer
Private m_NoOfMoves As Integer
Private m_IsInPlay As Boolean
Private m_name As String
Private m_key As String
Private m_IsCapturable As Boolean
Private m_valuePiece As Integer
Private m_OffsetPinAbsolute As Integer
Private m_OffsetPinRelative As Integer

Private m_RankPromotion As Integer
Private m_RankEnPassant As Integer

Private Sub Class_Initialize()
    m_LastMoveTurnNo = -1
    m_TurnNoPromotion = 0
    m_NoOfMoves = 0
    m_IsInPlay = True
    m_OffsetPinAbsolute = 0
    m_OffsetPinRelative = 0
End Sub

Public Sub BuildPiece(ByVal typeOfPiece As Byte, ByVal File As Byte, ByVal Rank As Byte, _
                      ByVal playerPiece As Player, ByVal valueOfPiece As Integer)
Dim sqr As Square

    m_typePiece = typeOfPiece
    m_valuePiece = valueOfPiece
    m_IsCapturable = m_typePiece <> typeKing
    Set m_player = playerPiece
    Set m_opponent = m_player.OtherPlayer
    Set m_board = m_player.BoardChess
    
    m_isWhite = m_player.IsWhite
    Set sqr = m_board.GetSquareFileRank(File, Rank)
    Set m_square = sqr
    Set m_squareInitial = sqr
    sqr.SetPieceSquare Me
    m_name = NamePiece(m_typePiece)
    m_key = m_name + m_square.Name
    m_player.ListPiece.Add Me, m_key
    
    If m_typePiece = typePawn Then
        m_RankPromotion = IIf(m_isWhite, rank7, rank2)
        m_RankEnPassant = IIf(m_isWhite, rank5, rank4)
    End If
    
End Sub

Public Property Get TypePiece() As Byte
    TypePiece = m_typePiece
End Property

Public Property Get ValuePiece() As Integer
    ValuePiece = m_valuePiece
End Property

Public Property Get LastMoveTurnNo() As Integer
    LastMoveTurnNo = m_LastMoveTurnNo
End Property

Public Property Let LastMoveTurnNo(ByVal Value As Integer)
    m_LastMoveTurnNo = Value
End Property

Public Property Get TurnNoPromotion() As Integer
    TurnNoPromotion = m_TurnNoPromotion
End Property

Public Property Get HasMoved() As Boolean
    HasMoved = (m_NoOfMoves <> 0) Or Not m_IsInPlay
End Property

Public Property Get IsWhite() As Boolean
    IsWhite = m_isWhite
End Property

Public Property Get IsCapturable() As Boolean
    IsCapturable = m_IsCapturable
End Property

Public Property Get HasBeenPromoted() As Boolean
    HasBeenPromoted = m_TurnNoPromotion > 0
End Property

Public Property Get IsInPlay() As Boolean
    IsInPlay = m_IsInPlay
End Property

Public Property Let IsInPlay(ByVal Value As Boolean)
    m_IsInPlay = Value
End Property

Public Property Get NoOfMoves() As Integer
    NoOfMoves = m_NoOfMoves
End Property

Public Property Let NoOfMoves(ByVal Value As Integer)
    m_NoOfMoves = Value
End Property

Public Sub NoOfMovesIncr()
    m_NoOfMoves = m_NoOfMoves + 1
End Sub

Public Sub NoOfMovesDecr()
    m_NoOfMoves = m_NoOfMoves - 1
    If m_NoOfMoves < 0 Then
        m_NoOfMoves = 0
    End If
End Sub

Public Property Get MovedLastMove() As Boolean
    MovedLastMove = False
End Property

Public Property Get Square() As Square
    Set Square = m_square
End Property

Public Sub SetSquare(ByVal sqr As Square)
    Set m_square = sqr
End Sub

Public Property Get SquareInitial() As Square
    Set SquareInitial = m_squareInitial
End Property

Public Property Get Name() As String
    If isFrench Then
        Name = Mid(strAbbreviationPiecesFr, m_typePiece, 1)
    Else
        Name = m_name
    End If
End Property

Public Property Get Player() As Player
    Set Player = m_player
End Property

Public Property Get Key() As String
    Key = m_key
End Property

Public Sub Capture()

    If m_typePiece = typePawn Then
        m_player.CountPawnDecr
    ElseIf m_IsCapturable Then
        m_player.CountNonPawnDecr
    End If
    m_opponent.ListCapture.Add Me, m_key
    m_player.ListPiece.Remove m_key
    m_square.SetPieceSquare Nothing
    m_IsInPlay = False
    
End Sub

Public Sub UncaptureNonPawn()

    m_player.ListPiece.Add Me, m_key, 1
    m_opponent.ListCapture.Remove m_key
    m_IsInPlay = True
    m_player.CountNonPawnIncr
    
End Sub

Public Sub UncapturePawn()

    m_player.ListPiece.Add Me, m_key
    m_opponent.ListCapture.Remove m_key
    m_IsInPlay = True
    m_player.CountPawnIncr
    
End Sub

Public Sub UncaptureNonPawnFen()

    If m_player.ListPiece.Count = 0 Then
        m_player.ListPiece.Add Me, m_key
    Else
        m_player.ListPiece.Add Me, m_key, 1
    End If
    m_opponent.ListCapture.Remove m_key
    m_IsInPlay = True
    m_player.CountNonPawnIncr
    
End Sub

Public Sub Promote(ByVal typePiecePromotion As Byte)

    If HasBeenPromoted Then ErrWarn 4800
    If m_typePiece <> typePawn Then ErrWarn 4810
    m_player.ListPiece.Remove m_key
    m_player.CountPawnDecr
    
    m_typePiece = typePiecePromotion
    m_name = NamePiece(m_typePiece)
    m_key = m_name + m_key
    m_player.ListPiece.Add Me, m_key, 1
    m_player.CountNonPawnIncr
    m_NoOfMoves = 0
    m_TurnNoPromotion = m_board.TurnNo

End Sub

Public Sub Demote()

    If Not HasBeenPromoted Then ErrWarn 4900
    m_player.CountNonPawnDecr
    m_player.ListPiece.Remove m_key
    
    m_typePiece = typePawn
    m_name = NamePiece(m_typePiece)
    m_player.CountPawnIncr
    m_key = Mid(m_key, 2)
    m_player.ListPiece.Add Me, m_key
    m_NoOfMoves = 5
    m_TurnNoPromotion = 0

End Sub

Public Sub GenerateLazyMoves(ByRef listMove As Collection)
    Select Case m_typePiece
    Case typePawn
        GenerateLazyMovesPawn listMove
        
    Case typeKnight
        GenerateLazyMovesKnight listMove
    
    Case typeBishop
        GenerateLazyMovesSliding nbrOffsetBishop, offsetBishop, listMove
        
    Case typeRook
        GenerateLazyMovesSliding nbrOffsetRook, offsetRook, listMove
    
    Case typeQueen
        GenerateLazyMovesSliding nbrOffsetQueen, offsetQueen, listMove
    
    Case typeKing
        GenerateLazyMovesKing listMove
      
    End Select
End Sub

Private Function IsPieceCapturable(ByVal sqr As Square) As Boolean
Dim pieceCapture As Piece

    If sqr Is Nothing Then
        IsPieceCapturable = False: Exit Function
    End If
    Set pieceCapture = sqr.PieceOnSquare
    If pieceCapture Is Nothing Then
        IsPieceCapturable = False: Exit Function
    End If
    If Not pieceCapture.IsCapturable Then
        IsPieceCapturable = False: Exit Function
    End If
    IsPieceCapturable = pieceCapture.IsWhite <> m_isWhite
End Function

Private Sub MoveAddSpecial(ByVal typeMove As Integer, ByVal sqrTo As Square, _
                            ByRef listMove As Collection)
Dim moveThis As New Move

    moveThis.BuildMove typeMove, Me, m_square, sqrTo, sqrTo.PieceOnSquare
    listMove.Add moveThis
End Sub

Private Sub MoveAddStandard(ByVal sqrTo As Square, ByRef listMove As Collection)
Dim moveThis As New Move

    moveThis.BuildMove moveStandard, Me, m_square, sqrTo, sqrTo.PieceOnSquare
    listMove.Add moveThis
End Sub

Private Sub MoveAddEnPassant(ByVal iOrdinal As Integer, ByVal pawnOpponent As Piece, _
                            ByRef listMove As Collection)
Dim moveThis As New Move, sqrTo As Square

    Set sqrTo = m_board.GetSquare(iOrdinal)
    moveThis.BuildMove moveEnPassant, Me, m_square, sqrTo, pawnOpponent
    listMove.Add moveThis
End Sub

Private Function isEnPassantCapture(ByVal iOrdinal As Integer, _
                                    ByRef pawnOpponent As Piece) As Boolean

    Set pawnOpponent = m_board.GetPawn(iOrdinal)
    If pawnOpponent Is Nothing Then
        isEnPassantCapture = False: Exit Function
    End If
    If pawnOpponent.IsWhite = m_isWhite Then
        isEnPassantCapture = False: Exit Function
    End If
    If pawnOpponent.NoOfMoves <> 1 Or pawnOpponent.LastMoveTurnNo <> m_board.TurnNo Then
        isEnPassantCapture = False: Exit Function
    End If
    isEnPassantCapture = True
    
End Function

Private Sub GenerateLazyMovesPawn(ByRef listMove As Collection)
Dim sqrTo As Square, iRankFrom As Integer, iOrdinal As Integer, typeMove As Integer
Dim pawnOpponent As Piece

    iRankFrom = m_square.Rank
    iOrdinal = m_square.Ordinal
    
    If iRankFrom = m_RankPromotion Then
        For typeMove = movePromotionQueen To movePromotionKnight
            Set sqrTo = m_board.GetSquare(iOrdinal + m_player.PawnAttackRightOffset)
            If IsPieceCapturable(sqrTo) Then
                MoveAddSpecial typeMove, sqrTo, listMove
            End If
            Set sqrTo = m_board.GetSquare(iOrdinal + m_player.PawnAttackLeftOffset)
            If IsPieceCapturable(sqrTo) Then
                MoveAddSpecial typeMove, sqrTo, listMove
            End If
            Set sqrTo = m_board.GetSquare(iOrdinal + m_player.PawnForwardOffset)
            MoveAddSpecial typeMove, sqrTo, listMove
        Next
    Else
        Set sqrTo = m_board.GetSquare(iOrdinal + m_player.PawnAttackRightOffset)
        If IsPieceCapturable(sqrTo) Then
            MoveAddStandard sqrTo, listMove
        End If
        Set sqrTo = m_board.GetSquare(iOrdinal + m_player.PawnAttackLeftOffset)
        If IsPieceCapturable(sqrTo) Then
            MoveAddStandard sqrTo, listMove
        End If
        Set sqrTo = m_board.GetSquare(iOrdinal + m_player.PawnForwardOffset)
        If Not sqrTo Is Nothing Then
            MoveAddStandard sqrTo, listMove
        End If
        If iRankFrom = m_RankEnPassant Then
            If isEnPassantCapture(iOrdinal + 1, pawnOpponent) Then
                MoveAddEnPassant iOrdinal + m_player.PawnAttackRightOffset, pawnOpponent, listMove
            End If
            If isEnPassantCapture(iOrdinal - 1, pawnOpponent) Then
                MoveAddEnPassant iOrdinal + m_player.PawnAttackLeftOffset, pawnOpponent, listMove
            End If
        ElseIf Not HasMoved And _
            m_board.IsSquareEmpty(iOrdinal + m_player.PawnForwardOffset) Then
            Set sqrTo = m_board.GetSquare(iOrdinal + m_player.PawnForward2Offset)
            If Not sqrTo Is Nothing Then
                MoveAddStandard sqrTo, listMove
            End If
        End If
    End If
End Sub

Private Sub GenerateLazyMovesKnight(ByRef listMove As Collection)
Dim sqrTo As Square, indOffset As Integer
Dim ordinalFrom As Byte, pieceCapture As Piece

    ordinalFrom = m_square.Ordinal
    For indOffset = 1 To nbrOffsetKnight
        Set sqrTo = m_board.GetSquare(ordinalFrom + offsetKnight(indOffset))
        If Not sqrTo Is Nothing Then
            Set pieceCapture = sqrTo.PieceOnSquare
            If pieceCapture Is Nothing Then
                MoveAddStandard sqrTo, listMove
            ElseIf pieceCapture.IsWhite <> m_isWhite And pieceCapture.IsCapturable Then
                MoveAddStandard sqrTo, listMove
            End If
        End If
    Next
End Sub

Private Sub GenSlidingMove(ByVal offsetDirection As Integer, listMove As Collection)
Dim sqrTo As Square, ordinalTo As Byte, pieceCapture As Piece

    ordinalTo = m_square.Ordinal + offsetDirection
    Set sqrTo = m_board.GetSquare(ordinalTo)
    Do While Not sqrTo Is Nothing
        Set pieceCapture = sqrTo.PieceOnSquare
        If pieceCapture Is Nothing Then
            MoveAddStandard sqrTo, listMove
        Else
            If pieceCapture.IsWhite <> m_isWhite And pieceCapture.IsCapturable Then
                MoveAddStandard sqrTo, listMove
            End If
            Exit Do
        End If
        ordinalTo = ordinalTo + offsetDirection
        Set sqrTo = m_board.GetSquare(ordinalTo)
    Loop
End Sub

Private Sub GenerateLazyMovesSliding(ByVal nbrOffset As Integer, _
               offsetPiece() As Integer, ByRef listMove As Collection)
Dim indOffset As Integer

    For indOffset = 1 To nbrOffset
        GenSlidingMove offsetPiece(indOffset), listMove
    Next
End Sub

Private Sub GenerateLazyMovesKing(ByRef listMove As Collection)
Dim sqrTo As Square, indOffset As Integer
Dim ordinalFrom As Byte

    ordinalFrom = m_square.Ordinal
    For indOffset = 1 To nbrOffsetKing
        Set sqrTo = m_board.GetSquare(ordinalFrom + offsetKing(indOffset))
        If m_player.CanKingGoTo(sqrTo) Then
            MoveAddStandard sqrTo, listMove
        End If
    Next
    If Not HasMoved Then
        If Not m_player.IsInCheck Then
            If m_player.CanCastleKingSide() Then
                Set sqrTo = m_board.GetSquare(ordinalFrom + 2)
                MoveAddSpecial moveCastleKingSide, sqrTo, listMove
            End If
            If m_player.CanCastleQueenSide() Then
                Set sqrTo = m_board.GetSquare(ordinalFrom - 2)
                MoveAddSpecial moveCastleQueenSide, sqrTo, listMove
            End If
        End If
    End If
End Sub

Public Function IsLegalMove() As Boolean
Dim moveList As New Collection, moveThis As Move, indMove As Integer

    GenerateLazyMoves moveList
    For indMove = 1 To moveList.Count
        Set moveThis = moveList(indMove)
        moveThis.DoMove ' Update IsInCheck
        moveThis.UndoMove
        If Not moveThis.IsInCheck Then
            IsLegalMove = True
            Exit Function
        End If
    Next
    IsLegalMove = False
End Function

Public Function IsDefending(ByVal ordinalTo As Byte) As Boolean
Dim offsetDirection As Integer, ordinalFrom As Byte, ordDiff As Integer
Dim isSqrDefended As Boolean

    isSqrDefended = False
    ordinalFrom = m_square.Ordinal
    ordDiff = offsetDiffOrdinal + ordinalTo - ordinalFrom
    Select Case m_typePiece
    Case typePawn
        If m_isWhite Then
            isSqrDefended = isWhitePawnAttacking(ordDiff)
        Else
            isSqrDefended = isBlackPawnAttacking(ordDiff)
        End If
        
    Case typeKnight
        isSqrDefended = isKnightAttacking(ordDiff)
    
    Case typeBishop
        offsetDirection = offsetBishopAttacking(ordDiff)
        If offsetDirection <> 0 Then
            isSqrDefended = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
        End If
        
    Case typeRook
        offsetDirection = offsetRookAttacking(ordDiff)
        If offsetDirection <> 0 Then
            isSqrDefended = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
        End If
    
    Case typeQueen
        offsetDirection = offsetQueenAttacking(ordDiff)
        If offsetDirection <> 0 Then
            isSqrDefended = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
        End If
            
    Case typeKing
        isSqrDefended = offsetKingAttacking(ordDiff)
    
    End Select
    IsDefending = isSqrDefended
End Function

Public Function IsAttacking(ByVal ordinalTo As Byte) As Boolean
Dim offsetDirection As Integer, ordinalFrom As Byte, ordDiff As Integer
Dim isSqrAttacked As Boolean

    isSqrAttacked = False
    ordinalFrom = m_square.Ordinal
    ordDiff = offsetDiffOrdinal + ordinalTo - ordinalFrom
    Select Case m_typePiece
    Case typePawn
        If m_isWhite Then
            isSqrAttacked = isWhitePawnAttacking(ordDiff)
        Else
            isSqrAttacked = isBlackPawnAttacking(ordDiff)
        End If
        
    Case typeKnight
        isSqrAttacked = isKnightAttacking(ordDiff)
    
    Case typeBishop
        offsetDirection = offsetBishopAttacking(ordDiff)
        If offsetDirection <> 0 Then
            isSqrAttacked = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
        End If
        
    Case typeRook
        offsetDirection = offsetRookAttacking(ordDiff)
        If offsetDirection <> 0 Then
            isSqrAttacked = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
        End If
    
    Case typeQueen
        offsetDirection = offsetQueenAttacking(ordDiff)
        If offsetDirection <> 0 Then
            isSqrAttacked = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
        End If
            
    Case typeKing
        isSqrAttacked = offsetKingAttacking(ordDiff)
    
    End Select
    IsAttacking = isSqrAttacked
End Function

Public Function IsCounterAttackPin(ByVal ordinalTo As Byte, ByVal offsetOfPin As Integer) As Boolean
Dim offsetDirection As Integer, ordinalFrom As Byte, ordDiff As Integer
Dim isSqrAttacked As Boolean

    isSqrAttacked = False
    ordinalFrom = m_square.Ordinal
    ordDiff = offsetDiffOrdinal + ordinalTo - ordinalFrom
    Select Case m_typePiece
    Case typePawn
        offsetDirection = offsetBishopAttacking(ordDiff)
        If offsetDirection <> 0 Then
            If offsetDirection = -offsetOfPin Then
                If m_isWhite Then
                    isSqrAttacked = isWhitePawnAttacking(ordDiff)
                Else
                    isSqrAttacked = isBlackPawnAttacking(ordDiff)
                End If
            End If
        End If
            
    Case typeBishop
        offsetDirection = offsetBishopAttacking(ordDiff)
        If offsetDirection <> 0 Then
            If offsetDirection = -offsetOfPin Then
                isSqrAttacked = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
            End If
        End If
        
    Case typeRook
        offsetDirection = offsetRookAttacking(ordDiff)
        If offsetDirection <> 0 Then
            If offsetDirection = -offsetOfPin Then
                isSqrAttacked = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
            End If
        End If
    
    Case typeQueen
        offsetDirection = offsetQueenAttacking(ordDiff)
        If offsetDirection <> 0 Then
            If offsetDirection = -offsetOfPin Then
                isSqrAttacked = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
            End If
        End If
            
    Case typeKnight, typeKing
        Stop
    
    End Select
    IsCounterAttackPin = isSqrAttacked
End Function

Public Function IsDefenseXray(ByVal ordinalTo As Byte, ByVal offsetDirXray As Integer) As Boolean
Dim offsetDirection As Integer, ordinalFrom As Byte, ordDiff As Integer
Dim isSqrAttacked As Boolean, typeSlider As Byte

    isSqrAttacked = False
    ordinalFrom = m_square.Ordinal
    If ordinalTo <> ordinalFrom Then
        ordDiff = offsetDiffOrdinal + ordinalTo - ordinalFrom
        Select Case m_typePiece
        Case typeBishop
            offsetDirection = offsetBishopAttacking(ordDiff)
            If offsetDirection = offsetDirXray Then
                isSqrAttacked = m_board.IsXRayEmpty(ordinalFrom, ordinalTo, offsetDirection, _
                    typeBishop, m_isWhite)
            End If
            
        Case typeRook
            offsetDirection = offsetRookAttacking(ordDiff)
            If offsetDirection = offsetDirXray Then
                isSqrAttacked = m_board.IsXRayEmpty(ordinalFrom, ordinalTo, offsetDirection, _
                    typeRook, m_isWhite)
            End If
        
        Case typeQueen
            offsetDirection = offsetQueenAttacking(ordDiff)
            If offsetDirection = offsetDirXray Then
                typeSlider = IIf(offsetDirection = offsetBishopAttacking(ordDiff), typeBishop, typeRook)
                isSqrAttacked = m_board.IsXRayEmpty(ordinalFrom, ordinalTo, offsetDirection, _
                    typeSlider, m_isWhite)
            End If
        End Select
    End If
    IsDefenseXray = isSqrAttacked
    
End Function

Public Function IsAttackXray(ByVal ordinalTo As Byte, ByVal offsetDirXray As Integer) As Boolean
Dim offsetDirection As Integer, ordinalFrom As Byte, isSqrAttacked As Boolean

    isSqrAttacked = False
    ordinalFrom = m_square.Ordinal
    If ordinalTo <> ordinalFrom Then
        Select Case m_typePiece
        Case typeBishop
            offsetDirection = offsetBishopAttacking(offsetDiffOrdinal + ordinalTo - ordinalFrom)
            If offsetDirection = offsetDirXray Then
                isSqrAttacked = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
            End If
            
        Case typeRook
            offsetDirection = offsetRookAttacking(offsetDiffOrdinal + ordinalTo - ordinalFrom)
            If offsetDirection = offsetDirXray Then
                isSqrAttacked = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
            End If
        
        Case typeQueen
            offsetDirection = offsetQueenAttacking(offsetDiffOrdinal + ordinalTo - ordinalFrom)
            If offsetDirection = offsetDirXray Then
                isSqrAttacked = m_board.IsRayEmpty(ordinalFrom, ordinalTo, offsetDirection)
            End If
        End Select
    End If
    IsAttackXray = isSqrAttacked
    
End Function

Private Sub TwinAdd(ByVal pieceThis As Piece, ByRef listTwin As Collection)
    If listTwin Is Nothing Then
        Set listTwin = New Collection
    End If
    listTwin.Add pieceThis
End Sub

Public Function TwinAligned(ByRef listTwin As Collection) As Boolean
Dim indPiece As Integer, pieceThis As Piece, pieces As Collection
Dim typeOfPiece As Byte, sqrTwin As Square
Dim ordinalRef As Integer, ordDiff As Integer

    If m_typePiece = typeKing Or m_typePiece = typePawn Then
        TwinAligned = False: Exit Function
    End If
    Set listTwin = Nothing
    Set pieces = m_player.ListPiece
    ordinalRef = m_square.Ordinal
    For indPiece = 1 To pieces.Count
        Set pieceThis = pieces(indPiece)
        typeOfPiece = pieceThis.TypePiece
        If typeOfPiece = typePawn Then
            Exit For
        End If
        If typeOfPiece = m_typePiece Then
            Set sqrTwin = pieceThis.Square
            ordDiff = sqrTwin.Ordinal - ordinalRef
            If ordDiff <> 0 Then
                ordDiff = ordDiff + offsetDiffOrdinal
                Select Case typeOfPiece
                Case typeKnight
                    If offsetKingAttacking(ordDiff) <> 0 Then
                        TwinAdd pieceThis, listTwin
                    End If
                    
                Case typeRook
                    If offsetRookAttacking(ordDiff) <> 0 Then
                        TwinAdd pieceThis, listTwin
                    End If
                
                Case typeBishop
                    If offsetBishopAttacking(ordDiff) <> 0 Then
                        TwinAdd pieceThis, listTwin
                    End If
                    
                Case typeQueen
                    If offsetQueenAttacking(ordDiff) <> 0 Then
                        TwinAdd pieceThis, listTwin
                    End If
                End Select
            End If
        End If
    Next
    TwinAligned = Not listTwin Is Nothing
End Function

Public Property Get OffsetPinAbsolute() As Integer
    OffsetPinAbsolute = m_OffsetPinAbsolute
End Property

Public Property Let OffsetPinAbsolute(ByVal Value As Integer)
    m_OffsetPinAbsolute = Value
End Property

Public Property Get OffsetPinRelative() As Integer
    OffsetPinRelative = m_OffsetPinRelative
End Property

Public Property Let OffsetPinRelative(ByVal Value As Integer)
    m_OffsetPinRelative = Value
End Property






